#ifndef	   IP_ADDRESS_H_INCLUDED
#define	   IP_ADDRESS_H_INCLUDED

#include  <string>
#include  <iostream>
#include  <cstdio>

extern "C"
{
	#include  <sys/types.h>
	#include  <sys/socket.h>
	#include  <netinet/in.h>
	#include  <netdb.h>
}

#ifdef HAVE_CONFIG_H
	#include  "config.h"
#else
	#if   defined( __FreeBSD__ )
		typedef unsigned long  in_addr_t;
		typedef unsigned short in_port_t;
	#elif defined( __linux )
		typedef unsigned long  in_addr_t;
		typedef int	       in_port_t;
	#endif
#endif

//
// Internet Protocol(V4) class
//
class  IP_Address
{
protected:
	in_addr_t	address;

public:
	IP_Address( const std::string &  host )
		: address(0)
		{
			struct hostent *	hp;
			if ( (hp = gethostbyname( host.c_str() ))  ==  NULL )
			{
#if 0
				std::perror( "gethostbyname" );
#elif 0
				using namespace std;
				perror( "gethostbyname" );
#else
				perror( "gethostbyname" );
#endif
			}

			memcpy( &address , hp -> h_addr , hp -> h_length );

			// <<TRANSLATE TO HOST BYTE ORDER>>
			// translate network byte order
			//   to machne depend(NOT NETWORK) byte order
			address = htonl( address );
		}

	IP_Address( unsigned char  a ,  unsigned char  b ,
		    unsigned char  c ,  unsigned char  d )
		: address(   ( static_cast<in_addr_t>(a) << (8 * 3) )
			   + ( static_cast<in_addr_t>(b) << (8 * 2) )
			   + ( static_cast<in_addr_t>(c) << (8 * 1) )
			   + ( static_cast<in_addr_t>(d) << (8 * 0) ) )
		{
		}

	IP_Address( in_addr_t  addr )
		: address(addr)
		{
		}

	IP_Address( const IP_Address &  ip )
		: address( ip.address )
		{
		}

	operator in_addr_t() const
		{
			return( address );
		}

};


inline
std::ostream &  operator<< ( std::ostream &  ostrm ,
			     const IP_Address &  addr )
{
	ostrm << ((addr >> (8*3)) & 0xff) << "."
	      << ((addr >> (8*2)) & 0xff) << "."
	      << ((addr >> (8*1)) & 0xff) << "."
	      << ((addr >> (8*0)) & 0xff);
	return( ostrm );
}


#endif	/* IP_ADDRESS_H_INCLUDED */
