#include  "soccer_option_analyzer.h"

#include  <iostream>
#include  <cstdlib>

extern "C"
{
#ifdef __linux
#include  <getopt.h>
#else
#include  <unistd.h>
#endif
}

using namespace std;

void   Soccer_Option_Analyzer::usage( ostream &  ostr ) const throw()
{
	ostr << "Usage : "
	     << (this -> argv_0)
	     << " [-h host] [-p port]"
	     << " [-t teammate] [-r player_number]" << endl;
}


Soccer_Option_Analyzer::Soccer_Option_Analyzer( int  argc ,  char  *argv[] ,
						string  default_team_name ,
						string  default_host_name ,
						in_port_t  default_port_number)
{
	this -> valid = true;

	this -> team = default_team_name;
	this -> host = default_host_name;
	this -> port = default_port_number;
	this -> reconnect_flag = false;
	this -> reconnect_player_num = 1;

	if ( argc == 0 )
	{
		this -> valid = false;
		return;
	}
	else
	{
		this -> argv_0 = argv[0];
	}

	int	ch;
	while( (ch = getopt( argc , argv , "h:p:t:r:" )) != -1 )
	{
		switch( ch )
		{
		case 'h':
			this -> host = optarg;
			break;

		case 'p':
		    {
			in_port_t	p;
			if ( (p = static_cast<in_port_t>(atoi(optarg))) == 0 )
			{
				this -> valid = false;
				return;
			}

			this -> port = p;
		    }
			break;

		case 't':
			this -> team = optarg;
			break;

		case 'r':
			reconnect_flag = true;

			if ( (reconnect_player_num = atoi(optarg)) == 0 )
			{
				this -> valid = false;
				return;
			}
			break;

		default:
			this -> valid = false;
			return;
			break;
		}
	}
}

Soccer_Option_Analyzer::~Soccer_Option_Analyzer()
{
}

Soccer_Option_Analyzer::operator bool() const throw()
{
	return( valid );
}

string  Soccer_Option_Analyzer::teamname() const throw()
{
	return( this -> team );
}

string  Soccer_Option_Analyzer::hostname() const throw()
{
	return( this -> host );
}

in_port_t  Soccer_Option_Analyzer::port_number() const throw()
{
	return( this -> port );
}

bool   Soccer_Option_Analyzer::reconnect() const throw()
{
	return( this -> reconnect_flag );
}

int    Soccer_Option_Analyzer::reconnect_player_number() const throw()
{
	if ( this -> reconnect_flag )
	{
		return( this -> reconnect_player_num );
	}
	else
	{
		return( -1 );
	}
}
