#include  "field_recog.h"
#include  "sight_info_analyzer.h"
#include  "s_basic.h"
#include  "ref_count_ptr.h"
#include  "d2_point.h"
#include  <cmath>
#include  <list>

using namespace std;

Debug_Info_from_Server	Field_Recog::latest_debug;
int			Field_Recog::latest_debug_time = 0;

Field_Recog::Field_Recog( const Game_Info &  g_info ,
			  const SServer_Param  &  param )
	: Field_Recog_Abstract( g_info , param ) ,
	  state( g_info.my_player_number ) ,
	  my_neck_angle( 0.0 ) ,
	  v_angle( View_Angle::Normal ) ,
	  v_quality( View_Quality::High ) ,
	  strict_ball_coordinate_and_vector_info_time( 0 ) ,
	  strict_ball_coordinate_info_time( 0 ) ,
	  strict_my_coordinate_and_angle_info_time( 0 ) ,
	  sight_count( 0 ) ,
	  ball_time_count( 0 ) ,
	  teammate_time_count( ELEVEN ) , opponent_time_count( ELEVEN ) ,
	  latest_sight() , latest_sight_time( 0 ) ,
	  body_info() , old_body_info() ,
	  latest_body() , latest_body_time( 0 ) ,
	  last_judgement() , last_judgement_t( -1 ) ,
	  play_mode_start_t( 0 ) ,
	  imaginary_ball_coordinate( D2_Vector::XY , 0.0 , 0.0 ) ,
	  imaginary_ball_vector( D2_Vector::XY , 0.0 , 0.0 )
{
	body_info.v_angle     = View_Angle::Normal;
	body_info.v_quality   = View_Quality::High;

	body_info.stamina     = this -> sserver_param().STAMINA_MAX();
	body_info.effort      = 1.0;
	body_info.speed       = 0.0;
	body_info.head_angle  = 0.0;

	body_info.n_kick      = 0;
	body_info.n_dash      = 0;
	body_info.n_turn      = 0;
	body_info.n_say       = 0;
	body_info.n_turn_neck = 0;

	old_body_info = body_info;
	latest_body   = body_info;

	if ( g_info.my_player_number >= 1 )
	{
		state.set_my_info(
			this -> sserver_param().initial_position(
						  g_info.our_side ,
						  g_info.my_player_number ) );
	}
}

Field_Recog::~Field_Recog()
{
}


void   Field_Recog::register_info( const Information_from_Server &  info )
	throw()
{
	try
	{
		if ( info.type() == Information_from_Server::Type_Sight )
		{
			register_sight_info( info.get_sight_info() ,
					     info.time() );
		}
		else if ( info.type() == Information_from_Server::Type_Body )
		{
			register_body_info( info.get_body_info() ,
					    info.time() );
		}
		else if ( info.type()
			  == Information_from_Server::Type_Whistle )
		{
			register_whistle_info( info.get_whistle_info() ,
					       info.time() );
		}
		else if ( info.type()
			  == Information_from_Server::Type_Debug )
		{
			register_debug_info( info.get_debug_info() ,
					     info.time() );
		}
	}
	catch(...)
	{
	}
}


void   Field_Recog::next_turn( const Soccer_Command &  prev ,
			       bool  last_command_accepted ) throw()
{
	// XXX
	(void)last_command_accepted;



	// My Coordinate and Angle
	if ( this -> current_time()
	     != this -> strict_my_coordinate_and_angle_info_time )
	{
		double	my_last_turn_speed = old_body_info.speed;
		if ( prev.type() == Soccer_Command::Dash )
		{
			my_last_turn_speed
				+= this -> sserver_param()
				     .dash_power_to_accel( prev.dash_power() );
		}

		// Coordinate
		try
		{
			D2_Vector	my_coordinate;
			double		my_body_angle;

			my_coordinate = state.my_region().barycenter();
			my_body_angle = state.my_body_angle();

			D2_Vector	vec( D2_Vector::Pole ,
					     my_last_turn_speed ,
					     my_body_angle );

			try
			{
				state.set_my_info( my_coordinate + vec ,
						   my_body_angle ,
						   state.my_face_angle() );
			}
			catch(...)
			{
				state.set_my_info( my_coordinate + vec ,
						   my_body_angle );
			}

		} catch(...){}

		// Angle
		if ( prev.type() == Soccer_Command::Turn )
		try
		{
			double	actual_angle = prev.turn_moment();

			actual_angle /=
			    ( 1.0 + this -> sserver_param().INERTIA_MOMENT()
				    * my_last_turn_speed );

			try
			{
				state.set_my_info( state.my_region() ,
						   state.my_body_angle()
						     + actual_angle ,
						   state.my_face_angle() );
			}
			catch(...)
			{
				state.set_my_info( state.my_region() ,
						   state.my_body_angle()
						     + actual_angle );
			}
		}
		catch(...) {}
	}

	// Coordinate
	if ( prev.type() == Soccer_Command::Move )
	{
		try
		{
			state.set_my_info( D2_Vector( prev.move_x() ,
						      prev.move_y() ) ,
					   state.my_body_angle() ,
					   state.my_face_angle() );
		}
		catch(...)
		{
			try
			{
				state.set_my_info( D2_Vector( prev.move_x() ,
							      prev.move_y() ) ,
						   state.my_body_angle() );
			}
			catch(...)
			{
				state.set_my_info( D2_Vector( prev.move_x() ,
							      prev.move_y() )
						   );
			}
		}
	}




	// Ball
#if 1
	if ( this -> current_time()
	       != this -> strict_ball_coordinate_and_vector_info_time )
	{
		// ball coordinate check
		D2_Vector	ball_coordinate;
		bool		have_ball_coordinate = false;
		try
		{
			ball_coordinate = state.ball_region().barycenter();

		} catch(...){}

		// ball vector check
		D2_Vector	ball_vector;
		bool		have_ball_vector = false;
		try
		{
			ball_vector = state.ball_vector().barycenter();

		} catch(...){}

		if ( have_ball_coordinate )
		{
			if ( have_ball_vector )
			{
#if 0
				// XXX: must check kickable or not.
				if ( prev.type() == Soccer_Command::Kick )
				try
				{
					double	vec_r;
					double	vec_theta;

					vec_r = this -> sserver_param()
						 .kick_power_to_accel(
						      prev.kick_power() ,
						      ... );

					prev.kick_direction();

					state.my_body_angle();

					ball_vector += vec;

				} catch(...){}
#endif

				if ( this -> current_time()
				  != this -> strict_ball_coordinate_info_time )
				{
					ball_coordinate += ball_vector;
				}

				ball_vector
				    *= this -> sserver_param().BALL_DECAY();

				state.set_ball_info( ball_coordinate ,
						     ball_vector );

				imaginary_ball_coordinate = ball_coordinate;
				imaginary_ball_vector     = ball_vector;
			}
			else
			{
				imaginary_ball_coordinate = ball_coordinate;
				imaginary_ball_vector
				    *= this -> sserver_param().BALL_DECAY();
			}
		}
		else
		{
			imaginary_ball_coordinate += imaginary_ball_vector;
			imaginary_ball_vector
			    *= this -> sserver_param().BALL_DECAY();
		}
	}
#else
	if ( this -> current_time()
	     != this -> strict_ball_coordinate_and_vector_info_time )
	{
		// XXX: should check last kick command
		try
		{
			D2_Vector	current_ball_point;
			current_ball_point = state.ball_region().barycenter();
			// if ball is not found, throw exception

			D2_Vector	next_ball_point;
			next_ball_point = current_ball_point
					  + state.ball_vector().barycenter();
			// if ball vector is unknown, throw exception

#if 0
			bool	ball_is_not_in_sight = false;

			try
			{
				D2_Vector	my_coordinate;
#if 0
				double		my_angle;
#endif

				my_coordinate = state.my_region().barycenter();
#if 0
				my_angle      = state.my_angle();
#endif

				bool	current_in_short_sensor
					= (current_ball_point
					   - my_coordinate).r()
					    <= this -> sserver_param()
						 .SHORT_SENSOR_LENGTH() * 0.9;

				bool	next_in_short_sensor
					= (next_ball_point - my_coordinate).r()
					    <= this -> sserver_param()
						 .SHORT_SENSOR_LENGTH() * 0.9;

				bool	in_short_sensor
					= ( current_in_short_sensor
					    || next_in_short_sensor );

#if 0
				bool	in_sight
				  = (   (next_point - my_coordinate).r()
				          <= 20.0
				     && fabs( (next_point - my_coordinate)
					       .theta() - my_angle)
					  <= 22.5 );

				ball_is_not_in_sight
					= (in_short_sensor || in_sight);
#else
				ball_is_not_in_sight = in_short_sensor;
#endif
			}
			catch(...)
			{
			}
#endif

#if 0
			if ( ball_is_not_in_sight )
			{
				D2_Region	no_region;

				state.set_ball_info( no_region );

				cout << "Ball Predict Cancelled!!" << endl;
			}
			else
#endif
			{
				D2_Region	new_ball_region(
							next_ball_point );
				imaginary_ball_coordinate = next_ball_point;

				D2_Region	new_ball_vector
					( state.ball_vector().barycenter()
					  * (this -> sserver_param())
					     .BALL_DECAY() );

				state.set_ball_info( new_ball_region ,
						     new_ball_vector );

				imaginary_ball_vector
					= new_ball_vector.barycenter();
			}
		}
		catch(...)
		{
		}

		// imaginary_ball_coordinate and vector
		try
		{
			imaginary_ball_coordinate
				= state.ball_region().barycenter();

			imaginary_ball_vector
				= state.ball_vector().barycenter();
		}
		catch(...)
		{
			imaginary_ball_coordinate += imaginary_ball_vector;
			imaginary_ball_vector
				*= this -> sserver_param().BALL_DECAY();
		}
	}
#endif
}


void   Field_Recog::turn_neck( double  moment ) throw()
{
	if ( moment > this -> sserver_param().MAX_NECK_MOMENT() )
	{
		moment = this -> sserver_param().MAX_NECK_MOMENT();
	}
	else if ( moment < this -> sserver_param().MIN_NECK_MOMENT() )
	{
		moment = this -> sserver_param().MIN_NECK_MOMENT();
	}

	my_neck_angle += moment;

	if ( my_neck_angle > this -> sserver_param().MAX_NECK_ANGLE() )
	{
		my_neck_angle = this -> sserver_param().MAX_NECK_ANGLE();
	}
	else if ( my_neck_angle < this -> sserver_param().MIN_NECK_ANGLE() )
	{
		my_neck_angle = this -> sserver_param().MIN_NECK_ANGLE();
	}

	// XXX
	try
	{
		this -> state.set_my_info( this -> state.my_region() ,
					   this -> state.my_body_angle() ,
					   my_neck_angle );
	} catch(...){}
}


SServer_Turn  Field_Recog::current_time() const throw()
{
	return( get_latest_body_time() );
}

const Game_Info &  Field_Recog::get_game_info() const throw()
{
	return( game_info );
}

const  Sight_Info_from_Server &  Field_Recog::get_latest_sight_info()
	const throw()
{
	return( latest_sight );
}


SServer_Turn  Field_Recog::get_latest_sight_time() const throw()
{
	return( latest_sight_time );
}


const Body_Info_from_Server &  Field_Recog::get_latest_body_info()
	const throw()
{
	return( latest_body );
}

SServer_Turn  Field_Recog::get_latest_body_time() const throw()
{
	return( latest_body_time );
}


const Field_State &  Field_Recog::get_field_state() const throw()
{
	return( state );
}


// protected
void   Field_Recog::register_sight_info(
			const Sight_Info_from_Server &  sight ,
			int  info_time ) throw()
{
	latest_sight_time = info_time;

#if 0
	try
	{
		latest_sight = sight;
		Sight_Info_Analyzer	s( latest_sight , my_neck_angle );
		state = s.get_field_state( game_info.my_player_number );
	}
	catch(...)
	{
		return;
	}
#endif
#if 1
	// XXX
	D2_Region	no_region;

#if 0
	if ( (++ ball_time_count) == 8 )
#else
	if ( (++ ball_time_count) == 3 )
#endif
	{
		state.set_ball_info( no_region );
	}

	for ( size_t  i = 0  ;  i < ELEVEN  ;  i ++ )
	{
		if ( (++ (teammate_time_count[i])) == 20 )
		{
			state.set_teammate_info( i + 1 , no_region );
		}

		if ( (++ (opponent_time_count[i])) == 20 )
		{
			state.set_opponent_info( i + 1 , no_region );
		}
	}


	Field_State	new_state( game_info.my_player_number ,
				   ELEVEN , ELEVEN );
	try
	{
		latest_sight = sight;

		Sight_Info_Analyzer	s( sight , my_neck_angle );

		new_state = s.get_field_state( game_info.my_player_number );
	}
	catch(...)
	{
		return;
	}

	// Set Ball Information
	try
	{
		try
		{
			// Coordinate and Vector
			D2_Vector	ball_vec;

#if 1
			try
			{
				ball_vec
				    = new_state.ball_vector().barycenter();
			}
			catch(...)
#endif
			{
				ball_vec = new_state.ball_region().barycenter()
					   - state.ball_region().barycenter();
			}

			if ( ball_vec.r() >=
			     this -> sserver_param().BALL_SPEED_MAX() * 2.0 )
			{
				ball_vec.set( D2_Vector::XY , 0.0 , 0.0 );
			}

			state.set_ball_info( new_state.ball_region() ,
					     ball_vec );

			try
			{
				imaginary_ball_coordinate
					= state.ball_region().barycenter();
				imaginary_ball_vector
					= state.ball_vector().barycenter();
			}
			catch(...)
			{
			}

			strict_ball_coordinate_info_time = info_time;
			strict_ball_coordinate_and_vector_info_time
				= info_time;
		}
		catch(...)
		{
			// Coordinate only
			state.set_ball_info( new_state.ball_region() );

			try
			{
				imaginary_ball_coordinate
					= state.ball_region().barycenter();
			}
			catch(...)
			{
			}

			strict_ball_coordinate_info_time = info_time;
		}

#if 1
		try
		{
			imaginary_ball_coordinate
				= state.ball_region().barycenter();
		} catch(...){}

		try
		{
			imaginary_ball_vector
				= state.ball_vector().barycenter();
		} catch(...){}
#endif

		ball_time_count = 0;
	}
	catch(...)
	{
	}

	for ( size_t  i = 1  ;  i <= ELEVEN  ;  i ++ )
	{
		try
		{
			D2_Region	r( new_state.teammate_region( i ) );

			try
			{
				double	body_angle
					= new_state.teammate_body_angle( i );
				try
				{
					double	face_angle
					  = new_state.teammate_face_angle( i );

					state.set_teammate_info( i , r ,
								 body_angle ,
								 face_angle );
				}
				catch(...)
				{
					state.set_teammate_info( i , r ,
								 body_angle );
				}
			}
			catch(...)
			{
				state.set_teammate_info( i , r );
			}

			teammate_time_count[i - 1] = 0;
		}
		catch(...)
		{
		}

		try
		{
			D2_Region	r( new_state.opponent_region( i ) );

			try
			{
				double	body_angle
					= new_state.opponent_body_angle( i );

				try
				{
					double	face_angle
					  = new_state.opponent_face_angle( i );

					state.set_opponent_info( i , r ,
								 body_angle ,
								 face_angle );
				}
				catch(...)
				{
					state.set_opponent_info( i , r ,
								 body_angle );
				}

			}
			catch(...)
			{
				state.set_opponent_info( i , r );
			}

			opponent_time_count[i - 1] = 0;
		}
		catch(...)
		{
		}
	}

	try
	{
		(void)state.my_region();
		(void)state.my_body_angle();
		(void)state.my_face_angle();
		strict_my_coordinate_and_angle_info_time = info_time;
	}
	catch(...)
	{
	}
#endif

	return;
}


// protected
void   Field_Recog::register_body_info(
			const Body_Info_from_Server &  body ,
			int  info_time ) throw()
{
	latest_body_time = info_time;

	old_body_info = body_info;
	body_info     = body;
	latest_body   = body;

	my_neck_angle = body.head_angle;

	// XXX
	try
	{
		this -> state.set_my_info( this -> state.my_region() ,
					   this -> state.my_body_angle() ,
					   my_neck_angle );
	} catch(...){}
}

// protected
void   Field_Recog::register_whistle_info(
			const Whistle_Info_from_Server &  whistle ,
			int  info_time ) throw()
{
	if ( whistle.type == Judgement_Type::One_of_Play_Modes )
	{
		this -> game_info.play_mode = whistle.play_mode;
		this -> play_mode_start_t   = info_time;

		switch( this -> game_info.play_mode )
		{
		case Play_Mode::Before_Kick_Off:
		case Play_Mode::Time_Over:

		case Play_Mode::Opponent_Kick_Off:
		case Play_Mode::Opponent_Kick_In:
		case Play_Mode::Opponent_Free_Kick:
		case Play_Mode::Opponent_Corner_Kick:
		case Play_Mode::Opponent_Goal:
		case Play_Mode::Opponent_Goal_Kick:

		case Play_Mode::Our_Kick_Off:
		case Play_Mode::Our_Kick_In:
		case Play_Mode::Our_Corner_Kick:
		case Play_Mode::Our_Goal:

		case Play_Mode::Our_Goal_Kick:
		case Play_Mode::Our_Free_Kick:
			try
			{
				state.set_ball_info( state.ball_region() ,
						     D2_Vector( 0.0 , 0.0 ) );
			} catch(...){}
			break;

		default:
			break;
		}
	}
	else
	{
		this -> last_judgement   = whistle;
		this -> last_judgement_t = info_time;

		if ( whistle.type == Judgement_Type::Goal )
		{
			if ( whistle.side == S_Side::Our_Side )
			{
				this -> game_info.our_point
					    = whistle.point;
			}
			else
			{
				this -> game_info.opponent_point
					    = whistle.point;
			}

			this -> game_info.play_mode
				= Play_Mode::Before_Kick_Off;
			this -> play_mode_start_t = info_time;
		}
	}
}

// protected
void   Field_Recog::register_debug_info(
			const Debug_Info_from_Server &  debug ,
			int  info_time ) throw()
{
	latest_debug_time = info_time;
	latest_debug      = debug;
}


Play_Mode  Field_Recog::play_mode() const throw()
{
	return( game_info.play_mode );
}

Whistle_Info_from_Server  Field_Recog::play_mode_reason() const throw()
{
	if ( this -> last_judgement_time() == this -> play_mode_start_time() )
	{
		return( last_judgement );
	}
	else
	{
		Whistle_Info_from_Server	unknown;

		return( unknown );
	}
}

int    Field_Recog::play_mode_start_time() const throw()
{
	return( this -> play_mode_start_t );
}

int    Field_Recog::last_judgement_time() const throw()
{
	return( this -> last_judgement_t );
}



SServer_Turn  Field_Recog::get_strict_ball_coordinate_and_vector_info_time()
	const throw()
{
	return( strict_ball_coordinate_and_vector_info_time );
}

SServer_Turn  Field_Recog::get_strict_ball_coordinate_info_time()
	const throw()
{
	return( strict_ball_coordinate_info_time );
}

SServer_Turn  Field_Recog::get_strict_my_coordinate_and_angle_info_time()
	const throw()
{
	return( strict_my_coordinate_and_angle_info_time );
}

int    Field_Recog::strict_ball_lose_turn( int  max_turn ) const throw()
{
	// XXX

	if ( this -> get_strict_ball_coordinate_and_vector_info_time()
	     != this -> current_time() )
	{
		return( 0 );
	}


	D2_Vector	ball_coordinate;
	D2_Vector	ball_vector;
	double		my_body_angle;

	try
	{
		ball_coordinate = this -> get_ball_coordinate();
		ball_vector     = this -> get_ball_vector();
		my_body_angle   = this -> get_my_body_angle();
	}
	catch(...)
	{
		return( 0 );
	}


	list<SObject_Player_Identifier>	check_list;

	SObject_Player_Identifier	id;
	for ( int  i = 1  ;  i <= static_cast<int>(ELEVEN)  ;  i ++ )
	{
		id.side = S_Side::Opponent_Side;
		id.player_number = i;

		check_list.push_back( id );

		if ( i  !=  this -> my_player_number() )
		{
			id.side = S_Side::Our_Side;

			check_list.push_back( id );
		}
	}

	for ( int  n = 0  ;  n <= max_turn  ;  n ++ )
	{
		double	kickable_max_dist
				= sserver_param().PLAYER_SPEED_MAX()
				    * static_cast<double>(n);

		list<SObject_Player_Identifier>::iterator  it
			= check_list.begin();

		while ( it != check_list.end() )
		{
			D2_Vector	ball = sserver_param()
						 .ball_basic_point_n_turn_later
						  ( ball_coordinate ,
						    ball_vector ,
						    n );

			try
			{
				double	d;
				d = (this -> get_player_coordinate( *it )
				     - ball).r();

				if ( d - (sserver_param().kickable_distance())
				     <= kickable_max_dist + 0.1 )
				{
					return( n );
				}
				else
				{
					double	k
						= sserver_param()
						  .PLAYER_SPEED_MAX()
						  * static_cast<double>(
							max_turn - n );

					double	b
						= sserver_param()
						  .BALL_SPEED_MAX()
						  * static_cast<double>(
							max_turn - n );

					if ( d >= k + b + 5.0 )
					{
						throw exception();
					}
				}

				it ++;
			}
			catch(...)
			{
				list<SObject_Player_Identifier>::iterator  er;
				er = it;
				it ++;

				check_list.erase(er);
			}
		}
	}

	return( max_turn + 1 );
}




//
// Encode
//
Field_Recog::operator std::string() const
{
	// XXX
	return( "" );
}


int    Field_Recog::my_player_number() const throw()
{
	return( game_info.my_player_number );
}


View_Angle  Field_Recog::view_angle() const throw()
{
	return( this -> v_angle );
}

View_Quality  Field_Recog::view_quality() const throw()
{
	return( this -> v_quality );
}


D2_Vector  Field_Recog::get_my_coordinate() const // throw(exception)
{
#if 0
	try
	{
#endif
		return( this -> state.my_region().barycenter() );
#if 0
	}
	catch(...)
	{
		throw exception();
	}
#endif
}

double  Field_Recog::get_my_body_angle() const // throw(exception)
{
	return( this -> state.my_body_angle() );
}

double  Field_Recog::get_my_face_angle() const // throw(std::exception)
{
#if 0
	return( this -> my_neck_angle );
#else
	return( this -> state.my_face_angle() );
#endif
}


D2_Vector  Field_Recog::get_ball_coordinate() const // throw(exception)
{
#if 0
	try
	{
#endif
		return( state.ball_region().barycenter() );
#if 0
	}
	catch(...)
	{
		throw exception();
	}
#endif
}

D2_Vector  Field_Recog::get_ball_relative() const // throw(exception)
{
#if 0
	try
	{
#endif
		return(   state.ball_region().barycenter()
		        - state.my_region().barycenter() );
#if 0
	}
	catch(...)
	{
		throw exception();
	}
#endif
}

D2_Vector  Field_Recog::get_ball_vector() const // throw(exception)
{
#if 0
	try
	{
#endif
		return( state.ball_vector().barycenter() );
#if 0
	}
	catch(...)
	{
		throw exception();
	}
#endif
}


D2_Vector   Field_Recog::get_player_coordinate
				( const SObject_Player_Identifier &  player )
					const // throw(std::exception)
{
	return( this -> get_player_coordinate( player.side ,
					       player.player_number ) );
}

D2_Vector   Field_Recog::get_player_coordinate( S_Side  side ,
						int  player_number)
	const // throw(exception)
{
#if 0
	try
	{
#endif
		return( state.player_region( side ,
					     player_number ).barycenter() );
#if 0
	}
	catch(...)
	{
		throw exception();
	}
#endif
}

D2_Vector  Field_Recog::get_teammate_coordinate( int  player_number )
	const // throw(std::exception)
{
#if 0
	try
	{
#endif
		return( state.teammate_region( player_number ).barycenter() );
#if 0
	}
	catch(...)
	{
		throw exception();
	}
#endif
}

D2_Vector  Field_Recog::get_opponent_coordinate( int  player_number )
	const // throw(std::exception)
{
#if 0
	try
	{
#endif
		return( state.opponent_region( player_number ).barycenter() );
#if 0
	}
	catch(...)
	{
		throw exception();
	}
#endif
}


double  Field_Recog::get_my_stamina() const throw()
{
	return( body_info.stamina );
}

double  Field_Recog::get_my_speed() const throw()
{
	return( body_info.speed );
}

const SServer_Param &  Field_Recog::sserver_param() const throw()
{
	return( sserver_parameter );
}


D2_Vector  Field_Recog::get_imaginary_ball_coordinate() const throw()
{
	try
	{
		return( this -> get_ball_coordinate() );
	}
	catch(...)
	{
		return( imaginary_ball_coordinate );
	}
}


D2_Vector  Field_Recog::get_imaginary_ball_vector() const throw()
{
	try
	{
		return( this -> get_ball_vector() );
	}
	catch(...)
	{
		return( imaginary_ball_vector );
	}
}




// class static
const Debug_Info_from_Server &  Field_Recog::server_debug_info() throw()
{
	return( latest_debug );
}

// class static
int  Field_Recog::server_debug_info_time() throw()
{
	return( latest_debug_time );
}
