#ifndef	   FIELD_RECOG_ABSTRACT_H_INCLUDED
#define	   FIELD_RECOG_ABSTRACT_H_INCLUDED

#include  "game_info.h"
#include  "sserver_param.h"
#include  "information_from_server.h"
#include  "soccer_command.h"
#include  "field_state.h"
#include  "sserver_turn.h"

class  Field_Recog_Abstract
{
protected:
	Game_Info		game_info;
	const SServer_Param &	sserver_parameter;

public:
	 Field_Recog_Abstract( const Game_Info &  g_info ,
			       const SServer_Param  &  ss_param )
		 : game_info( g_info ) , sserver_parameter( ss_param ){}

virtual	~Field_Recog_Abstract(){}

//
// protocol between Soccer_Interface and Field_Recog
//
virtual	void	register_info( const Information_from_Server & info )
							throw() = 0;
virtual	void	next_turn( const Soccer_Command &  prev ,
			   bool  last_command_accepted = true ) throw() = 0;

virtual	void	turn_neck( double  moment ) throw() = 0;

virtual	const SServer_Param &		sserver_param() const throw() = 0;

virtual	SServer_Turn			current_time() const throw() = 0;

virtual	const Sight_Info_from_Server &	get_latest_sight_info()
							const throw() = 0;
virtual	SServer_Turn			get_latest_sight_time()
							const throw() = 0;
virtual	const Body_Info_from_Server &	get_latest_body_info()
							const throw() = 0;
virtual	SServer_Turn			get_latest_body_time()
							const throw() = 0;

virtual	const Field_State &		get_field_state() const throw() = 0;

virtual	Play_Mode			play_mode() const throw() = 0;
virtual Whistle_Info_from_Server	play_mode_reason() const throw() = 0;
virtual	int				play_mode_start_time()
							const throw() = 0;
virtual	int				last_judgement_time()
							const throw() = 0;

virtual	int		my_player_number() const throw() = 0;

virtual	View_Angle	view_angle() const throw() = 0;
virtual	View_Quality	view_quality() const throw() = 0;

virtual	SServer_Turn	get_strict_ball_coordinate_and_vector_info_time()
							const throw() = 0;
virtual	SServer_Turn	get_strict_my_coordinate_and_angle_info_time()
							const throw() = 0;

virtual	int		strict_ball_lose_turn( int  max_turn = 10 )
							const throw() = 0;

	//
	// Encode
	//
virtual	operator std::string() const = 0;




	//
	// some short cuts
	//
virtual	D2_Vector  get_my_coordinate()   const /*throw(std::exception)*/ = 0;
virtual	double	   get_my_body_angle()   const /*throw(std::exception)*/ = 0;
virtual	double	   get_my_face_angle()   const /*throw(std::exception)*/ = 0;
virtual	D2_Vector  get_ball_coordinate() const /*throw(std::exception)*/ = 0;
virtual	D2_Vector  get_ball_relative()   const /*throw(std::exception)*/ = 0;
virtual	D2_Vector  get_ball_vector()     const /*throw(std::exception)*/ = 0;
virtual	D2_Vector  get_player_coordinate
				( const SObject_Player_Identifier & )
					const /*throw(std::exception)*/ = 0;
virtual	D2_Vector	get_player_coordinate( S_Side , int )
					const /*throw(std::exception)*/ = 0;
virtual	D2_Vector	get_teammate_coordinate( int )
					const /*throw(std::exception)*/ = 0;
virtual	D2_Vector	get_opponent_coordinate( int )
					const /*throw(std::exception)*/ = 0;

virtual	double		get_my_stamina() const throw() = 0;
virtual	double		get_my_speed()   const throw() = 0;

virtual	D2_Vector	get_imaginary_ball_coordinate() const throw() = 0;
virtual	D2_Vector	get_imaginary_ball_vector()     const throw() = 0;
};


#endif	/* FIELD_RECOG_ABSTRACT_H_INCLUDED */
