#ifndef	   SOCCER_INTERFACE_H_INCLUDED
#define	   SOCCER_INTERFACE_H_INCLUDED

#include  "s_basic.h"
#include  "soccer_command.h"
#include  "soccer_composite_command.h"
#include  "sserver_connection_with_game_info.h"
#include  "field_recog.h"
#include  "field_state.h"
#include  "sserver_param.h"
#include  "ref_count_ptr.h"
#include  <vector>

class  Soccer_Interface
{
public:
	struct  Auto_Change_View_Type
	{
		enum  Type { Manual , Narrow_High };
		Auto_Change_View_Type::Type	type;
	};

protected:
	SServer_Connection_with_Game_Info &	server_connection;
	const SServer_Param &			params;

	std::vector< ref_count_ptr<Field_Recog_Abstract> >	field_recog;

	Soccer_Command		prev_main_command;

	SServer_Turn		last_sight_info_turn;
	SServer_Turn		last_body_info_turn;

	Time_Stamp		last_sight_info_time_stamp;
	Time_Stamp		last_body_info_time_stamp;

	Auto_Change_View_Type	auto_change_view_type;
	View_Angle		auto_change_view_narrow_high_view_mode;

	bool			game_over_flag;

protected:
static	const	long	BEGINNING_OF_TURN_MAX_WAIT_TIME = 10 * 1000; //usec

	void	register_info( const Information_from_Server &  info ,
			       const Time_Stamp &  stamp ) throw();

public:
	 Soccer_Interface( SServer_Connection_with_Game_Info & ,
			   const SServer_Param & );
	~Soccer_Interface();


	//
	// Register Field_Recog
	//
	void	register_field_recog( ref_count_ptr<Field_Recog_Abstract>  fr )
								      throw();


	//
	// Wait and Get Infomation
	//
	void	wait_for_next_info( Information_from_Server::Info_Type  t )
								      throw();

	void	wait_for_next_turn(
		    long  max_wait = BEGINNING_OF_TURN_MAX_WAIT_TIME ) throw();

	void	wait_for_next_sight_info() throw();


	//
	// Automatic Change_View Setting
	//
	void	set_automatic_change_view( Auto_Change_View_Type::Type  t )
								throw();

	//
	// Server Parameter
	//
	const SServer_Param &	get_sserver_param() const throw();


	//
	// playing game now or not.
	//
	bool	game_is_over() const throw();

	//
	// server respond or not.
	//
	bool	server_no_response() const throw();


	//
	// Server Raw Connection
	//
	int	send_init(
			const std::string &  team_name ,
			bool  goalie = false ,
			Game_Info *  g_info = NULL ,
			int  major_version
			  = SServer_Connection::SSERVER_DEFAULT_MAJOR_VERSION ,
			int  minor_version
			  = SServer_Connection::SSERVER_DEFAULT_MINOR_VERSION )
			  throw();

	int	send_reconnect( const std::string &  team_name ,
				int  player_number ,
				Game_Info *  g_info ,
				long  interval_usec = 0 ) throw();

	int	send_bye( long  interval_usec = 0 ) throw();

	int	send_move( double  x ,  double  y ,
			   long  interval_usec = 0 ) throw();
	int	send_dash( double  power ,  long  interval_usec = 0 ) throw();

	int	send_kick( double  power ,  double  direction ,
			   long  interval_usec = 0 ) throw();
	int	send_turn( double  moment ,
			   long  interval_usec = 0 ) throw();
	int	send_say( const std::string &  message ,
			  long  interval_usec = 0 ) throw();
	int	send_catch( double  direction ,
			    long  interval_usec = 0 ) throw();
	int	send_change_view( View_Angle  angle ,  View_Quality  quality ,
				  long  interval_usec = 0 ) throw();
	int	send_sense_body( long  interval_usec = 0 ) throw();
	int	send_turn_neck( double  moment ,
				long  interval_usec = 0 ) throw();

	int	send_command( const Soccer_Command &  command ,
			      long  interval_usec = 0 )	throw();

	int	send_reconnect( const std::string &  team_name ,
				int  player_number ,
				bool  default_wait ) throw();

	int	send_bye( bool  default_wait ) throw();

	int	send_move( double  x ,  double  y ,
			   bool  default_wait ) throw();
	int	send_dash( double  power ,  bool  default_wait ) throw();
	int	send_kick( double  power ,  double  direction ,
			   bool  default_wait ) throw();
	int	send_turn( double  moment ,  bool  default_wait ) throw();
	int	send_say( const std::string &  message ,
			  bool  default_wait ) throw();
	int	send_catch( double  direction ,  bool  default_wait ) throw();
	int	send_change_view( View_Angle  angle ,  View_Quality  quality ,
				  bool  default_wait ) throw();
	int	send_sense_body( bool  default_wait ) throw();
	int	send_turn_neck( double  direction ,
				bool  default_wait ) throw();

	int	send_command( const Soccer_Command &  command ,
			      bool  default_wait ) throw();


	//
	// Server Connection
	//
	int	send_composite_command
			( const Soccer_Composite_Command &  composite_command ,
			  long  interval_usec ) throw();


	int	send_composite_command
			( const Soccer_Composite_Command &  composite_command ,
			  bool  default_wait ) throw();
};


#endif	/* SOCCER_INTERFACE_H_INCLUDED */
