#ifndef	   INFORMATION_FROM_SERVER_H_INCLUDED
#define	   INFORMATION_FROM_SERVER_H_INCLUDED

#include  "s_basic.h"


///////////////////////////////////////////////////////////////////////////////
//			      Body_Info					     //
///////////////////////////////////////////////////////////////////////////////
struct  Body_Info_from_Server
{
	View_Angle	v_angle;
	View_Quality	v_quality;

	double	stamina;
	double	effort;

	double	speed;

	double	head_angle;

	int	n_kick;
	int	n_dash;
	int	n_turn;
	int	n_say;
	int	n_turn_neck;

	Body_Info_from_Server()
		: v_angle( View_Angle::Normal ) ,
		  v_quality( View_Quality::High ) ,
		  stamina(0.0) , effort(0.0) ,
		  speed(0.0) ,
		  head_angle(0.0) ,
		  n_kick(0) , n_dash(0) , n_turn(0) , n_say(0) , n_turn_neck(0)
		{
		}
};



///////////////////////////////////////////////////////////////////////////////
//			     Whistle_Info				     //
///////////////////////////////////////////////////////////////////////////////
struct  Whistle_Info_from_Server
{
	Judgement_Type	type;

	Play_Mode	play_mode;
	S_Side		side;
	int		player_number;
	int		point;

	Whistle_Info_from_Server()
		: type( Judgement_Type::Unknown ) ,
		  play_mode( Play_Mode::Unknown ) ,
		  side( S_Side::Unknown ) , player_number( 0 ) , point( 0 )
		{
		}
};


///////////////////////////////////////////////////////////////////////////////
//			   Audio_Info					     //
///////////////////////////////////////////////////////////////////////////////
#include  <string>

struct  Audio_Info_from_Server
{
	typedef  enum { ST_Self , ST_Player }  Sender_Type;

	double		direction;
	Sender_Type	sender;
	std::string	message;

	Audio_Info_from_Server()
		: direction( 0.0 ) , sender( ST_Self ) , message()
		{
		}
};



///////////////////////////////////////////////////////////////////////////////
//			      Sight_Info				     //
///////////////////////////////////////////////////////////////////////////////

#include  "d2_vector.h"
//
//  Object Location
//
struct  SObject_Locational_Info_from_Server
{
	bool	has_distance;
	bool	has_direction;
	bool	has_d_distance;
	bool	has_d_direction;
	bool	has_body_direction;
	bool	has_face_direction;

	double	distance;
	double	direction;
	double	d_distance;
	double	d_direction;
	double	body_direction;
	double	face_direction;

	bool	in_sight;

	SObject_Locational_Info_from_Server()
		: has_distance( false ) , has_direction( false ) ,
		  has_d_distance( false ) , has_d_direction( false ) ,
		  has_face_direction( false ) ,
		  distance(0.0) , direction(0.0) ,
		  d_distance(0.0) , d_direction(0.0) ,
		  body_direction(0.0) , face_direction(0.0) ,
		  in_sight( true )
		{
		}

	operator D2_Vector() const
		{ return( D2_Vector( D2_Vector::Pole ,
				     distance , direction ) ); }
};


//
//  Object and Locational Information
//
struct  Ball_Info_from_Server
{
	SObject_Ball_Identifier			entity;
	SObject_Locational_Info_from_Server	relative;
};

struct  Flag_Info_from_Server
{
	SObject_Flag_Identifier			entity;
	SObject_Locational_Info_from_Server	relative;
};

struct  Line_Info_from_Server
{
	SObject_Line_Identifier			entity;
	SObject_Locational_Info_from_Server	relative;
};

struct  Goal_Info_from_Server
{
	SObject_Goal_Identifier			entity;
	SObject_Locational_Info_from_Server	relative;
};

struct  Player_Info_from_Server
{
	SObject_Player_Identifier		entity;
	SObject_Locational_Info_from_Server	relative;
};

//
//  Sight_Info_from_Server
//
#include  <vector>

struct  Sight_Info_from_Server
{
	std::vector<   Ball_Info_from_Server >	ball;
	std::vector< Player_Info_from_Server >	player;
	std::vector<   Goal_Info_from_Server >	goal;
	std::vector<   Flag_Info_from_Server >	flag;
	std::vector<   Line_Info_from_Server >	line;

	 Sight_Info_from_Server()
		 : ball() , player() , goal() , flag() , line()
		{
		}

#ifdef LIB_MEMORY
	~Sight_Info_from_Server()
		{
			ball.clear();
			player.clear();
			goal.clear();
			flag.clear();
			line.clear();
		}
#endif
};


///////////////////////////////////////////////////////////////////////////////
//			      Error_Info				     //
///////////////////////////////////////////////////////////////////////////////
#include  <string>

struct  Error_Info_from_Server
{
	std::string	error_message;	// XXX

	Error_Info_from_Server() : error_message() {}
};


///////////////////////////////////////////////////////////////////////////////
//			      OK_Info					     //
///////////////////////////////////////////////////////////////////////////////
struct  OK_Info_from_Server
{
	// XXX
	typedef  enum { Move , Look , Check_Ball ,
			Say , Ear , Chamge_Mode } OK_Type;
	OK_Type		type;

	typedef  enum { In_Field , Out_of_Field ,
			In_Our_Goal , In_Oppornent_Goal } Ball_Position;

	Ball_Position	ball_position;

	bool	ear;

	OK_Info_from_Server()
		: type( Move ) , ball_position( In_Field ) , ear( true )
		{
		}
};



///////////////////////////////////////////////////////////////////////////////
//			      Debug_Info				     //
///////////////////////////////////////////////////////////////////////////////
struct  Debug_Info_from_Server
{
	bool		have_true_my_coordinate;
	D2_Vector	true_my_coordinate;

	bool		have_true_my_body_angle;
	double		true_my_body_angle;

	bool		have_true_my_face_angle;
	double		true_my_face_angle;

	Debug_Info_from_Server()
		: have_true_my_coordinate( false ) ,
		  true_my_coordinate( 0.0 , 0.0 ) ,
		  have_true_my_body_angle( false ) ,
		  true_my_body_angle( 0.0 ) ,
		  have_true_my_face_angle( false ) ,
		  true_my_face_angle( 0.0 )
	{
	}
};



///////////////////////////////////////////////////////////////////////////////
//		       Information_from_Server				     //
///////////////////////////////////////////////////////////////////////////////
class  Information_from_Server
{
public:
	typedef  enum {
		Type_Unknown ,
		Type_Sight , Type_Audio , Type_Whistle , Type_Body ,
		Type_OK ,
		Type_Error ,
		Type_No_Info ,
		Type_Debug
	} Info_Type;

protected:
	struct Initialized
	{
		bool	true_or_false;
		Initialized() : true_or_false( false ) {}
	};
	Initialized			initialized;

	Info_Type			type_of_info;
	int				time_of_info;
	std::string			raw_mes;

	Sight_Info_from_Server *	sight;
	Audio_Info_from_Server *	audio;
	Whistle_Info_from_Server *	whistle;
	Body_Info_from_Server *		body;
	Error_Info_from_Server *	err;
	Debug_Info_from_Server *	debug;

public:
	 Information_from_Server();
	 Information_from_Server( const Information_from_Server & );
	~Information_from_Server();

	Information_from_Server &  operator=
					( const Information_from_Server & );

	void	reset_info(); // to Type_Unknown

	void	set_raw_message( const string &  mes );
	void	set_time( int  t );

	void	set_info( const Sight_Info_from_Server &  si );
	void	set_info( const Audio_Info_from_Server &  au );
	void	set_info( const Whistle_Info_from_Server &  wi );
	void	set_info( const Body_Info_from_Server &  bd );
	void	set_info( const Error_Info_from_Server &  er );
	void	set_info( const Debug_Info_from_Server &  de );
	void	set_no_info(); // to Type_No_Info

public:
	std::string	raw_message() const;

	Info_Type	type() const;
	int		time() const;
	bool		illegal() const;
	bool		no_info() const;

	Sight_Info_from_Server		get_sight_info() const;
	Audio_Info_from_Server		get_audio_info() const;
	Whistle_Info_from_Server	get_whistle_info() const;
	Body_Info_from_Server		get_body_info() const;
	Error_Info_from_Server		get_error_info() const;
	Debug_Info_from_Server		get_debug_info() const;
};


#endif	/* INFORMATION_FROM_SERVER_H_INCLUDED */
