#ifndef	   SOCCER_COMMAND_H_INCLUDED
#define	   SOCCER_COMMAND_H_INCLUDED

#include  "s_basic.h"
#include  <string>

struct  Soccer_Command
{
public:
	typedef  enum { Move , Dash , Kick , Turn , Catch ,
			Turn_Neck , Change_View , Sense_Body , Say ,
			Wait , No_Command , Illegal } Command_Type;

protected:
	Command_Type	command;

	double		arg1;
	double		arg2;
	std::string	mes;
	View_Angle	view_angle;
	View_Quality	view_quality;

public:
	 Soccer_Command()
		: command( No_Command ) ,
		  arg1( 0.0 ) , arg2( 0.0 ) ,
		  mes() ,
		  view_angle( View_Angle::Normal ) ,
		  view_quality( View_Quality::High )
		{
		}

virtual	~Soccer_Command()
		{
		}

virtual	Command_Type	type() const
		{
			return( command );
		}

virtual	Soccer_Command &  move( double  x ,  double  y )
		{
			command = Move;
			arg1    = x;
			arg2    = y;
			return( *this );
		}

virtual	Soccer_Command &  dash( double  power )
		{
			command = Dash;
			arg1    = power;
			return( *this );
		}

virtual	Soccer_Command &  kick( double  power ,  double  direction )
		{
			command = Kick;
			arg1    = power;
			arg2    = direction;
			return( *this );
		}

virtual	Soccer_Command &  turn( double  moment )
		{
			command = Turn;
			arg1    = moment;
			return( *this );
		}


virtual	Soccer_Command &  catch_command( double  direction )
		{
			command = Catch;
			arg1    = direction;
			return( *this );
		}

virtual	Soccer_Command &  turn_neck( double  direction )
		{
			command = Turn_Neck;
			arg1 = direction;
			return( *this );
		}

virtual	Soccer_Command &  change_view( View_Angle  angle ,
				       View_Quality  quality )
		{
			command      = Change_View;
			view_quality = quality;
			view_angle   = angle;

			return( *this );
		}

virtual	Soccer_Command &  sense_body()
		{
			command = Sense_Body;
			return( *this );
		}

virtual	Soccer_Command &  say( const std::string &  m )
		{
			command     = Say;
			this -> mes = m;
			return( *this );
		}

virtual	Soccer_Command &  wait()
		{
			command = Wait;
			return( *this );
		}

virtual	Soccer_Command &  no_command()
		{
			command = No_Command;
			return( *this );
		}

virtual	Soccer_Command &  illegal()
		{
			command = Illegal;
			return( *this );
		}

virtual	operator Soccer_Command::Command_Type() const
		{
			return( command );
		}

virtual	double	move_x() const
		{
			return( arg1 );
		}

virtual	double	move_y() const
		{
			return( arg2 );
		}

virtual	double	dash_power() const
		{
			return( arg1 );
		}

virtual	double	kick_power() const
		{
			return( arg1 );
		}

virtual	double	kick_direction() const
		{
			return( arg2 );
		}

virtual	double	turn_moment() const
		{
			return( arg1 );
		}

virtual	double	catch_direction() const
		{
			return( arg1 );
		}

virtual	double	turn_neck_direction() const
		{
			return( arg1 );
		}

virtual	View_Angle	change_view_view_angle() const
		{
			return( view_angle );
		}

virtual	View_Quality	change_view_view_quality() const
		{
			return( view_quality );
		}

virtual	std::string	say_message() const
		{
			return( mes );
		}

};


#if 0
class  Move_Command : public Soccer_Command
{
public:
	Move_Command( double  x ,  double  y )
		{ this -> move( x , y ); }
};

class  Dash_Command : public Soccer_Command
{
public:
	Dash_Command( double  power )
		{ this -> dash( power ); }
};

class  Kick_Command : public Soccer_Command
{
public:
	Kick_Command( double  power ,  double  direction )
		{ this -> kick( power , direction ); }
};

class  Turn_Command : public Soccer_Command
{
public:
	Turn_Command( double  moment )
		{ this -> turn( moment ); }
};

class  Catch_Command : public Soccer_Command
{
public:
	Catch_Command( double  direction )
		{ this -> catch_command( direction ); }
};

class  Turn_Neck_Command : public Soccer_Command
{
public:
	Turn_Neck_Command( double  direction )
		{ this -> turn_neck( direction ); }
};

class  Change_View_Command : public Soccer_Command
{
public:
	Change_View_Command( View_Angle  angle ,  View_Quality  quality )
		{ this -> change_view( angle , quality ); }
};

class  Sense_Body_Command : public Soccer_Command
{
public:
	Sense_Body_Command()
		{ this -> sense_body(); }
};

class  Say_Command : public Soccer_Command
{
public:
	Say_Command( const std::string &  m )
		{ this -> say( m ); }
};

class  Wait_Command : public Soccer_Command
{
public:
	Wait_Command()
		{ this -> wait(); }
};

class  No_Command : public Soccer_Command
{
public:
	No_Command()
		{ this -> no_command(); }
};

class  Illegal_Command : public Soccer_Command
{
public:
	Illegal_Command()
		{ this -> illegal(); }
};
#endif


#endif	/* SOCCER_COMMAND_H_INCLUDED */
