#ifndef	   SSERVER_CONNECTION_H_INCLUDED
#define	   SSERVER_CONNECTION_H_INCLUDED

// Author:		H. Shimora
// Last-Modified:	Aug 11 1998
// Version:		0.10

///-----------------------------------------------
/// Change Log:
///-----------------------------------------------
// version 0.10  Aug 11 1998
//
//


#include  "sserver_io.h"
#include  "information_from_server.h"
#include  "game_info.h"
#include  "time_stamp.h"
#include  "soccer_command.h"
#include  "ip_address.h"
#include  <string>
#include  <cstddef>

class  SServer_Connection
{
public:
	static	const	in_port_t	SSERVER_DEFAULT_PORT
					= SServer_Socket::SSERVER_DEFAULT_PORT;

	static	const	long		TURN_INTERVAL = 100 * 1000; // 100 msec

	static	const	int		SSERVER_DEFAULT_MAJOR_VERSION = 5;
	static	const	int		SSERVER_DEFAULT_MINOR_VERSION = 0;

protected:
	SServer_IO *		serv_io;
	bool			serv_io_must_delete;

	std::string		team_name;
	S_Side_LR		our_side;

	Time_Stamp		send_time;

	bool			use_send_radian;
	bool			use_recv_radian;

	long			default_wait_time;

public:
	 SServer_Connection( const std::string &  host = "localhost" ,
			     in_port_t  port = SSERVER_DEFAULT_PORT ,
			     bool  use_send_radian = true ,
			     bool  use_recv_radian = true ,
			     long  default_wait_time = TURN_INTERVAL ,
			     long  max_response = 0 );

	 SServer_Connection( SServer_IO *  io ,
			     bool  use_send_radian = true ,
			     bool  use_recv_radian = true ,
			     long  default_wait_time = TURN_INTERVAL );

virtual	~SServer_Connection();

	// Don't allow this.
	 SServer_Connection( const SServer_Connection & );
	// Don't allow this.
	 SServer_Connection &  operator=( const SServer_Connection & );

virtual	operator bool();

virtual	bool	respond();

virtual	int		send( const std::string &  str ,
			      long  interval_usec = 0 );
virtual	std::string	recv( bool  block );
virtual	std::string	recv( long  usec );

virtual	long		default_wait() const;

virtual	int	send_init(
		    const std::string &  team_name ,
		    bool  goalie = false ,
		    Game_Info *  g_info = NULL ,
		    int  major_version = SSERVER_DEFAULT_MAJOR_VERSION ,
		    int  minor_version = SSERVER_DEFAULT_MINOR_VERSION );

virtual	int	send_init( const std::string &  team_name ,
			   bool  goalie ,
			   Game_Info *  g_info );

virtual	int	send_reconnect( const std::string &  team_name ,
				int  player_number ,
				Game_Info *  g_info = NULL ,
				long  interval_usec = 0 );

virtual	int	send_bye( long  interval_usec = 0 );

virtual	int	send_move( double  x ,  double  y ,  long  interval_usec = 0 );
virtual	int	send_dash( double  power ,  long  interval_usec = 0 );

virtual	int	send_kick( double  power ,  double  direction ,
			   long  interval_usec = 0 );
virtual	int	send_turn( double  moment ,  long  interval_usec = 0 );
virtual	int	send_say( const std::string &  message ,
			  long  interval_usec = 0 );
virtual	int	send_catch( double  direction ,  long  interval_usec = 0 );
virtual	int	send_change_view( View_Angle  angle ,  View_Quality  quality ,
				  long  interval_usec = 0 );
virtual	int	send_sense_body( long  interval_usec = 0 );
virtual	int	send_turn_neck( double  direction ,  long  interval_usec = 0 );

virtual	int	send_command( const Soccer_Command &  command ,
			      long  interval_usec = 0 );

#if 0
	//
	// coach commands
	//
virtual	int	send_look( long  interval_usec = 0 );
virtual	int	send_check_ball( long  interval_usec = 0 );
virtual	int	send_ear( bool  on_off , long  interval_usec = 0 );
virtual	int	send_change_mode( Play_Mode  mode , long  interval_usec = 0 );
#endif

virtual	int	send_reconnect( const std::string &  team_name ,
				int  player_number ,
				Game_Info *  g_info ,
				bool  default_wait );

virtual	int	send_bye( bool  default_wait );

virtual	int	send_move( double  x ,  double  y ,  bool  default_wait );
virtual	int	send_dash( double  power ,  bool  default_wait );
virtual	int	send_kick( double  power ,  double  direction ,
			   bool  default_wait );
virtual	int	send_turn( double  moment ,  bool  default_wait );
virtual	int	send_say( const std::string &  message ,  bool  default_wait );
virtual	int	send_catch( double  direction ,  bool  default_wait );
virtual	int	send_change_view( View_Angle  angle ,  View_Quality  quality ,
				  bool  default_wait );
virtual	int	send_sense_body( bool  default_wait );
virtual	int	send_turn_neck( double  direction ,  bool  default_wait );

virtual	int	send_command( const Soccer_Command &  command ,
			      bool  default_wait );

virtual	Information_from_Server		recv_info( bool  block = false );
virtual	Information_from_Server		recv_info( long  usec );
virtual	Information_from_Server		recv_info( int  usec );
};


#endif	/* SSERVER_CONNECTION_H_INCLUDED */
